/**
 * @file 工作流 步骤
 * @module step
 * @author r2space@gmail.com
 * @version 1.0.0
 */

'use strict';


export default class Step extends React.Component {

  constructor(props) {
    super(props);
    this.state = {
      activities: this.props.activities,
      state: this.props.state
    };
  }

  render() {

    const steps = [];
    this.state.activities.forEach(item => {

      const state = this.getState(item);
      if (state.action === 'none') {
        return null;
      }

      steps.push(dom('div', {className: 'item'},
        dom('p', null, state.name + ' ' + Step.ACTION[state.action]),
        dom('p', {className: 'time'}, state.time),
        state.photo
          ? this.renderImageStep(state.photo, item.id)
          : this.renderIconStep(state.icon, item.id)
        )
      );
      steps.push(dom('i', {className: 'step-arrow arrow-type1'}));
    });

    // 去除最后一个右向箭头
    steps.pop();

    return dom('div', {className: 'step-container'},
      dom('div', {className: 'step'}, steps),
      this.renderComment(),
      this.renderAction()
    );
  }

  renderImageStep(image, activity) {
    return dom('img', {className: 'img-circle', activity: activity, src: this.props.imageURL + image});
  }

  renderIconStep(icon, activity) {
    return dom('div', {className: 'div-circle', activity: activity},
      dom('i', {className: `fa ${icon}`})
    )
  }

  renderComment() {

    if (this.state.state !== 'processing') {
      return null;
    }

    return dom('div', {className: 'step-comment'},
      dom('form', {className: 'light-form'},
        dom('section', null,
          dom('label', {className: 'label'}, '审批意见'),
          dom('div', {className: 'input'},
            dom('input', {type: 'text'})
          )
        )
      )
    );
  }

  renderAction() {
    return dom('div', {className: 'step-action'},
      dom('button', {type: 'button', className: 'btn btn-sm btn-default', onClick: this.props.onCancel}, '取消'),
      this.state.state === 'processing' ?
        dom('button', {
          type: 'button',
          className: 'btn btn-sm btn-primary pull-right',
          style: {marginLeft: '5px'},
          onClick: this.props.onApprove
        }, '同意') : null,
      this.state.state === 'processing' ?
        dom('button', {
          type: 'button',
          className: 'btn btn-sm btn-warning pull-right',
          onClick: this.props.onDeny
        }, '拒绝') : null,
      this.state.state === 'init' ?
        dom('button', {
          type: 'button',
          className: 'btn btn-sm btn-primary pull-right',
          onClick: this.props.onCreate
        }, '创建') : null,
      this.state.state === 'terminate' ?
        dom('button', {
          type: 'button',
          className: 'btn btn-sm btn-warning pull-right',
          onClick: this.props.onTerminate
        }, '终止') : null
    );
  }

  componentDidUpdate() {

    const activities = this.state.activities
      , actionDetail = this.getActionDetail;

    $('.img-circle,.div-circle').each(function () {
      $(this).popover({
        placement: 'right',
        title: '审批详细',
        html: true,
        content: actionDetail(activities.find(item => item.id === $(this).attr('activity')))
      });
    });
  }

  getActionDetail(activity) {

    const memo = activity.performer.map(item => {
      const time = item.time ? moment(item.time).format('YYYY-MM-DD HH:mm') : ''
        , action = Step.ACTION[item.action];
      return `${item.name} ${action} ${time} ${item.memo}`;
    });

    return memo.join('<hr>');
  }

  getState(activity) {

    const deny = activity.performer.filter(item => item.action === 'deny');
    if (deny.length > 0) {
      return Object.assign({action: 'deny'}, this.getPerformer(deny));
    }

    if (activity.isActive) {
      return Object.assign({action: 'processing'}, this.getPerformer(activity.performer));
    }

    const create = activity.performer.filter(item => item.action === 'create');
    if (create.length > 0) {
      return Object.assign({action: 'create'}, this.getPerformer(create));
    }

    const approve = activity.performer.filter(item => item.action === 'approve');
    if (approve.length > 0) {
      return Object.assign({action: 'approve'}, this.getPerformer(approve));
    }

    const none = activity.performer.filter(item => item.action === 'none');
    if (none.length > 0) {
      return Object.assign({action: 'none'}, this.getPerformer(none));
    }
  }

  getPerformer(performers) {

    const [performer] = performers;

    // 设置审批者为图标，区分单人和多人
    let icon = performers.length === 1 ? 'fa-user' : 'fa-users'
      , name = performer.name + (performers.length === 1 ? '' : '等')
      , time = performer.time ? moment(performer.time).format('YYYY-MM-DD HH:mm') : '-'
      , photo = '';

    // 审批者为一个人时，显示头像
    if (performers.length === 1) {
      if (performer.photo && performer.photo.length > 0) {
        photo = performer.photo[0].path;
      }
    }

    return {icon: icon, photo: photo, name: name, time: time};
  }

  setStep(activities, state) {
    this.setState({activities: activities, state: state});
  }
}

Step.ACTION = {processing: '审批中', create: '起草', deny: '拒绝', approve: '同意', none: '待审批'};

Step.defaultProps = {
  activities: [],
  imageURL: '/api/file/image?id=',
  state: 'init',      // 流程状态
  showFuture: false,  // 是否显示未来步骤
  onCreate: () => {   // 创建
  },
  onCancel: () => {   // 取消
  },
  onApprove: () => {  // 同意
  },
  onDeny: () => {     // 拒绝
  },
  onTurndown: () => { // 退回，驳回
  },
  onTerminate: () => {// 终止
  }
};

Step.init = (node, option) => {
  return ReactDOM.render(
    React.createElement(Step, option),
    document.getElementById(node)
  );
};
